@@ -28,7 +28,9 @@ mod json;
 mod lazy_data;
 mod middleware_from_fn;
 mod ndjson;
+mod query;
 mod redirect;
+mod spa;
 mod utils;
 
 // public API
